/*
 * ALSA Patch Bay
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *
 * You have permission to use this file under the GNU General
 * Public License, version 2 or later.  See the file COPYING
 * for the full text.
 *
 */

#include "apb.h"
#include <stdlib.h>

#include "driver/apb-alsa.h"
#include "gui/apb-gtkmm.h"

APB::DebugLevel debugLevel = APB::DEBUG;

namespace APB {

static void * xmalloc (size_t size) {
  void * ptr;
  ptr = malloc (size);
  if (!ptr) {
    exit (1);
  }
  return ptr;
}

static void * xrealloc (void * mem, size_t size) {
  void * ptr;
  ptr = realloc (mem, size);
  if (!ptr) {
    exit (1);
  }
  return ptr;
}


string Misc::int2string (int i) {
  static uint size = 16;
  static char * s = NULL;
  int err = -1;

  if (!s) s = (char *) xmalloc (size * sizeof(char));

  while (err == -1) {
    err = snprintf (s, size, "%d", i);
    if (err == -1) {
      size += 16;
      s = (char *) xrealloc (s, size);
    }
  }

  return string (s);
}

int Misc::randomNumber (double lower, double upper) {
  return (int) ((lower-1)+(int) ((upper+1)*rand()/(RAND_MAX+1.0)));
}


/////////////////////////////////////////
//////////// AlsaPatchBay ///////////////
/////////////////////////////////////////


Driver * AlsaPatchBay::newDriver (string title, int argc, char ** argv,
                                         DriverType type, void * data) {
  switch (type) {
    case ALSA:
      return new Alsa::Driver (title, argc, argv, data);
      break;
  }
}

UI * AlsaPatchBay::newUI (string title, int argc, char ** argv,
                                 UIType type, void * data, Driver * driver) {
  switch (type) {
    case GTKMM:
      return new GTKmm::UI (title, argc, argv, data, driver);
      break;
  }
}

} /* namespace APB */

int main(int argc, char ** argv) {
  srand (time(0));
  using namespace APB;
  AlsaPatchBay * apb;
  try {
    apb = new AlsaPatchBay ("ALSA Patch Bay", argc, argv);
  } catch (Exception e) {
    cerr << endl << "Caught exception while creating AlsaPatchBay: " << e.desc << endl;
    return 1;
  }
  apb->run ();
  return 0;
}

