/*
 * ALSA Patch Bay
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *
 * You have permission to use this file under the GNU General
 * Public License, version 2 or later.  See the file COPYING
 * for the full text.
 *
 */

#ifndef __APB_ALSA_H__
#define __APB_ALSA_H__

#include <list>
#include <string>
#include <alsa/asoundlib.h>

#include "apb.h"

namespace APB {
namespace Alsa {

class Driver;

class Addr : public APB::Addr {
  private:
    snd_seq_addr_t _addr;
    Driver * _driver;
  public:
    Addr (const snd_seq_addr_t& addr, Driver * driver)
     : _addr (addr), _driver (driver) {}
    Addr (const Addr& addr)
     : _addr (addr._addr), _driver (addr._driver) {}
    const snd_seq_addr_t& addr () const { return _addr; }
//    void addr (const snd_seq_addr_t& addr) { _addr = addr; }
    int client () const { return _addr.client; }
    int port () const { return _addr.port; }
    virtual bool equals (const APB::Addr * addr) const {
      if (!addr) {
        cerr << DEBUG_STRING << "addr is null; returning false" << endl;
        return false;
      }
      return ((Alsa::Addr *)addr)->_addr.client == _addr.client
             && ((Alsa::Addr *)addr)->_addr.port == _addr.port;
    }
    virtual APB::Driver * driver () const { return (APB::Driver *) _driver; }
    virtual string getName () const;
};

class Driver : public APB::Driver {
  private:
    snd_seq_t * _seq;
    list<APB::Addr *> _readPorts;
    list<APB::Addr *> _writePorts;
    list<APB::Subscription> _subscriptions;
    bool our_seq;
    string _title;
    bool _jfd;
    void refreshPorts (APB::UI&, list<APB::Addr *>&, unsigned int);
    void doPortSubscription (APB::UI&,
                             snd_seq_port_subscribe_t *,
                             const APB::Addr *,
                             const APB::Addr *);
    void doPortUnsubscription (APB::UI&,
                             snd_seq_port_subscribe_t *,
                             const APB::Addr *,
                             const APB::Addr *);
    void subscribePorts (APB::UI&,
                         const APB::Addr *,
                         const APB::Addr *,
                         bool);
  public:
    Driver (string, int, char **, void * = 0);
    Driver (const Driver& driver)
      : _seq (driver._seq),
        _readPorts (driver._readPorts),
        _writePorts (driver._writePorts),
        _subscriptions (driver._subscriptions) {}
    virtual ~Driver () {
      if (our_seq) {
        // close alsa
      }
    }
    virtual string findClientName (const APB::Addr *) const;
    virtual string findPortName (const APB::Addr *) const;
    virtual const list<APB::Addr *>& getReadPorts (APB::UI&);
    virtual const list<APB::Addr *>& getWritePorts (APB::UI&);
    virtual const list<APB::Subscription>& getSubscriptions (APB::UI&);
    virtual void refreshPorts (APB::UI&);
    virtual void refreshSubscriptions (APB::UI&);
    virtual void subscribePorts (APB::UI&, const APB::Addr *, const APB::Addr *);
    virtual void unsubscribePorts (APB::UI&, const APB::Addr *, const APB::Addr *);
    virtual void setJFD (bool);
};

} /* namespace Alsa */
} /* namespace APB */

#endif /* __APB_ALSA_H__ */
